<?php
/*
Plugin Name: Link File Info
Description: Display file type and size after links to local media files in posts.  Requires PHP 5.  Only applies for full posts, not comments or post excerpts .  Parses every post through DOMDocument on view, so unlikely to be wildly efficient.  Link parsing might be a bit rough - needs more extensive testing.  Invokes '/usr/bin/file' to figure out file types, which gives very pretty and accurate output, but again probably isn't very efficient.
Version: 0.3
Author: Tim Serong
Author URI: http://www.wirejunkie.com/
*/

/*
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.
*/

if ( !class_exists( 'LinkFileTypesPlugin' ) ) {

	class LinkFileTypesPlugin
	{
		private $host;

		function LinkFileTypesPlugin() {
			register_activation_hook( __FILE__, array( &$this, 'activation_hook' ) );
			add_action( 'admin_init', array( &$this, 'admin_init') );
			add_filter( 'the_content', array( &$this, 'the_content' ) );

			// Get hostname of current site for use in URL parsing
			// (is there a better way to do this?)
			$this->host = substr( strstr( get_option( 'siteurl' ), '//' ), 2 );
		}

		// Cribbed from http://us.php.net/manual/en/function.filesize.php,
		// comment from "nak5ive at DONT-SPAM-ME dot gmail dot com" dated
		// 11-Jun-2009 05:59
		function format_bytes ( $bytes, $precision = 1 ) {
			$units = array( 'Bytes', 'KB', 'MB', 'GB', 'TB' );

			$bytes = max( $bytes, 0 );
			$pow = floor( ( $bytes ? log( $bytes ) : 0 ) / log( 1024 ) );
			$pow = min( $pow, count( $units ) - 1 );

			$bytes /= pow( 1024, $pow);

			return round( $bytes, $precision) . ' ' . $units[$pow];
		}

		function activation_hook() {
			if ( version_compare( PHP_VERSION, '5.0.0', '<' ) || !class_exists('domDocument') ) {
				// Have to force deactivate, because the plugin has
				// already been set active at this point.
				deactivate_plugins(__FILE__);
				trigger_error('This plugin requires PHP 5 or newer with the DOM extension (http://www.php.net/manual/en/book.dom.php)', E_USER_ERROR );
			}

			add_option( 'lfi_path_to_file_cmd', '/usr/bin/file' );
		}

		function admin_init() {
			// Add this plugin's settings to the media page
			add_settings_section( 'link-file-info', 'Link File Info Settings', array( &$this, 'settings_section' ), 'media' );
			add_settings_field( 'lfi_path_to_file_cmd', 'Path to \'file\' command', array( &$this, 'settings_field_path_to_file_cmd' ), 'media', 'link-file-info' );			
			register_setting( 'media', 'lfi_path_to_file_cmd' );
		}

		function settings_section() {
?>
<p>These settings control how file metadata is inserted into posts by the Link File Info plugin.</p>
<?php
		}

		function settings_field_path_to_file_cmd() {
?>
<input id="lfi_path_to_file_cmd" name="lfi_path_to_file_cmd" type="text" value="<?php echo form_option( 'lfi_path_to_file_cmd' ); ?>" /> (executed to determine link file type)
<?php
		}

		function the_content( $content ) {
			$file_cmd = get_option( 'lfi_path_to_file_cmd' );
			if ( !is_executable( $file_cmd ) ) {
				error_log( "link-file-info plugin not processing post ('$file_cmd' is not executable)" );
				return $content;
			}

			$dom = new domDocument;
			$dom->loadHTML( $content );
			$doc  = $dom->documentElement;
			$links = $dom->getElementsByTagName( 'a' );

			foreach ( $links as $link )
			{
				$href = $link->getAttribute( 'href' );
				if ( !$href )
					continue;

				$url = parse_url( $href );
				if ( !array_key_exists( 'host', $url ) ||		// relative URL, or...
					 $url['host'] == $this->host ) {		// ...absolute URL on our site

					$file_path = path_join( ABSPATH, ltrim( $url['path'], '/' ) );
					if ( file_exists( $file_path ) ) {
						// Bingo!
						$type = exec( escapeshellcmd( $file_cmd ) . ' -b ' . escapeshellarg( $file_path ) );
						$size = filesize( $file_path );
						if ( $type && $size ) {
							$description = $dom->createTextNode( ' (' . $type . ', ' . $this->format_bytes( $size ) . ') ' );
							if ( $link == $link->parentNode->lastChild )
								$link->parentNode->appendChild( $description );
							else
								$link->parentNode->insertBefore( $description, $link->nextSibling );
						}
					}
				} else {
					if ( isset( $_GET['lfi_debug'] ) )
						echo "\n<!--\nlink-file-info ignoring link '$href':\n" . print_r($url, true) . " -->\n";
				}
			}

			// We end up with implicit HTML & BODY elements, need to
			// get rid of this before output.  Emitting only the BODY
			// element gets us part the way there...
			$body = $doc->getElementsByTagName( 'body' )->item(0);
			$html = $dom->saveXML( $body );
			// ...now $html is "<body>....</body>", so return everything
			// between the first '>' and last '<' characters.
			$first = strpos( $html, '>' );
			$last  = strrpos( $html, '<' );
			$first++;	// Start one past the first '>'.
			return substr( $html, $first, $last - $first );
		}
	}
}

if ( class_exists( 'LinkFileTypesPlugin' ) ) {
	$link_file_types_plugin = new LinkFileTypesPlugin();
}

?>
